/**************************************************************************** 
** File: tospingd.c
**
** Author: Michael S. Borella <mike_borella@mw.3com.com>
**
** TOSping server
**
*****************************************************************************/

#include <stdio.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>
#include "tosping.h"
#include "parse_cl.h"
#include "ip.h"

struct arg_t *my_args;     /* Command line arguments */
extern char version[];

/*----------------------------------------------------------------------------
 *
 * main()
 *
 *----------------------------------------------------------------------------
 */

int main(int argc, char *argv[])
{
  const int one = 1;
  int s;
  IPHdr iph;
  char buf[32];
  struct sockaddr_in sin;    /* an Internet endpoint address         */
  void syserr(char *,...);
  u_int32_t temp_ip;
  int tos;
  int alen;
  int n;

  /*
   * Announce who we are
   */

  printf("%s v%s  3Com Advanced Technologies Research Center\n", 
          argv[0], version);

  /*
   * Parse command line
   */
  
  my_args = parse_cl(argc, argv);
 
  /*
   * Set up raw socket (leave root mode)
   */
  
  s = socket(AF_INET, SOCK_RAW, IPPROTO_TOS);
  setuid(getuid());
  if (setsockopt(s, IPPROTO_IP, IP_HDRINCL, (const void *) &one, 
		 sizeof(one)) < 0)
    syserr("Can't put raw socket into header-write mode");

  while(1)
    {
      if (recvfrom(s, buf, sizeof(buf), 0, (struct sockaddr *) &sin, 
		   (int *) &alen) < 0)
	syserr("recvfrom error");

      /*
       * Copy the IP header into the structure
       */
      
      memcpy((char *) &iph, buf, sizeof(iph));
      
#ifdef DEBUG
      {
	struct in_addr in;
	
	in.s_addr = iph.ip_src;
	printf("DEBUG: received packet with TOS = %d from %s\n", iph.ip_tos,
	       inet_ntoa(in));
      }
#endif

      /*
       * Read the TOS field, then change the packet so that we can
       * transmit it
       */
      
      tos = iph.ip_tos;
      iph.ip_tos = my_args->t;
      iph.ip_id = 0; /* tells kernel to make one up */
      iph.ip_ttl = 255;
      iph.ip_p = IPPROTO_TOS;
      iph.ip_csum = 0; /* kernel should compute this for us */
      temp_ip = iph.ip_src;
      iph.ip_src = iph.ip_dst;
      iph.ip_dst = temp_ip;

#ifdef DEBUG
      {
	struct in_addr in;

	in.s_addr = iph.ip_dst;
	printf("DEBUG: sending packet with TOS = %d to %s\n", iph.ip_tos,
	       inet_ntoa(in));
      }
#endif

      /*
       * Write the IP header to the buffer
       */

       memcpy (buf, (char *) &iph, sizeof(iph));
       n = sprintf(&buf[sizeof(iph)], "%d:%d", tos, my_args->t);

       /*
	* For some reason, the source address isn't written correctly
	* into sin (is this a raw socket thing?).  So we'll fix that
	* here...
	*/

       sin.sin_family = AF_INET;
       sin.sin_addr.s_addr = iph.ip_dst;

       /*
	* Send it!
	*/

       if (sendto(s, buf, sizeof(iph)+n, 0, (struct sockaddr *) &sin, 
		  sizeof(sin)) < 0)
	   syserr("sendto error");
    }
  
  return 0;

}

